@@ -148,3 +148,22 @@ pub async fn initiate_payment_handler<T: Context + Send>(
 
     Ok(co)
 }
+
+pub async fn initiate_order_handler<T: Context + Send>(
+    ctx: &mut T,
+    checkout_id: &str,
+    args: Option<<T as PaymentProcessor>::ProcessArgs>,
+) -> Result<Checkout, Error> {
+    let maybe_co = ctx.get_checkout_for_update(&checkout_id).await?;
+    if maybe_co.is_none() {
+        return Err(new_not_found_error(
+            "the requested checkout could not be found",
+        ));
+    }
+
+    let mut co = maybe_co.unwrap();
+
+    co.initiate_order(ctx, args).await?;
+
+    Ok(co)
+}
