@@ -1,5 +1,5 @@
 use async_trait::async_trait;
-use chrono::{DateTime, Duration, Utc};
+use chrono::{DateTime, Utc};
 use schemars::JsonSchema;
 use serde::{Deserialize, Serialize};
 use uuid::Uuid;
@@ -11,6 +11,7 @@ use crate::inventory::ReserveResult;
 use crate::invoice::Invoice;
 use crate::item::Item;
 use crate::money::{Currency, Money};
+use crate::order::Order;
 use crate::payment::handlers as payment_handlers;
 use crate::payment::{Initiator, PaymentProcessor};
 use crate::shipping::{FulfillmentType, FulfillmentTypeSelection, ShippingQuote};
@@ -20,9 +21,9 @@ pub mod handlers;
 #[derive(Serialize, Deserialize, JsonSchema)]
 pub enum State {
     Shopping,                         // transitions to ItemsConfirmed
-    ItemsConfirmed(DateTime<Utc>),    // transitions to Shopping or PaymentInProgress
+    ItemsConfirmed(DateTime<Utc>),    // transitions to Shopping, PaymentInProgress or Completed
     PaymentInProgress(DateTime<Utc>), // can only enter this state from ItemsConfirmed (before deadline expiry). not allowed to free items due to deadline expiry in this state, unless payment is cancelled. transitions to either Shopping (if payment is cancelled) or Completed
-    Completed, // can only enter this state from PaymentInProgress. this is a terminal state
+    Completed, // can only enter this state from ItemsConfirmed or PaymentInProgress. this is a terminal state
 }
 
 #[derive(Serialize, Deserialize, JsonSchema)]
@@ -132,7 +133,6 @@ impl Checkout {
     }
 
     async fn enter_completed_state(&mut self) -> Result<(), Error> {
-        // TODO: create new order in the PendingConfirmation or Confirmed state and set checkout's order_id field
         Ok(self.state = State::Completed)
     }
 }
@@ -154,14 +154,7 @@ impl Checkout {
         ctx: &mut C,
     ) -> Result<(), Error> {
         match self.state {
-            State::ItemsConfirmed(deadline) => {
-                if deadline - Utc::now() >= Duration::minutes(60) {
-                    return Ok(());
-                }
-
-                ctx.free_items(&self.items).await?;
-                self.enter_items_confirmed_state(ctx).await
-            }
+            State::ItemsConfirmed(_) => Ok(()),
             State::Shopping => self.enter_items_confirmed_state(ctx).await,
             _ => Err(new_invalid_state_error(
                 "must be in the Shopping or ItemsConfirmed state",
@@ -177,6 +170,14 @@ impl Checkout {
         match self.state {
             State::PaymentInProgress(_) => Ok(()),
             State::ItemsConfirmed(deadline) => {
+                // check for contact information
+                if self.contact.is_none() {
+                    return Err(new_application_error(
+                        "MISSING_CONTACT_INFORMATION",
+                        "please provide contact information to continue",
+                    ));
+                }
+
                 // check for a fulfillment type
                 if self.fulfillment_type.is_none() {
                     return Err(new_application_error(
@@ -185,13 +186,8 @@ impl Checkout {
                     ));
                 }
 
-                if deadline - Utc::now() >= Duration::minutes(60) {
-                    return self
-                        .enter_payment_in_progress_state(ctx, args, deadline)
-                        .await;
-                }
-
-                Err(new_application_error("EXPIRED_DEADLINE", "the checkout completion deadline has been exceeded, please reconfirm items and try again"))
+                self.enter_payment_in_progress_state(ctx, args, deadline)
+                    .await
             }
             _ => Err(new_invalid_state_error(
                 "must be in the ItemsConfirmed or PaymentInProgress state",
@@ -202,6 +198,25 @@ impl Checkout {
     async fn ensure_completed_state(&mut self) -> Result<(), Error> {
         match self.state {
             State::Completed => Ok(()),
+            State::ItemsConfirmed(_) => {
+                // check for contact information
+                if self.contact.is_none() {
+                    return Err(new_application_error(
+                        "MISSING_CONTACT_INFORMATION",
+                        "please provide contact information to continue",
+                    ));
+                }
+
+                // check for a fulfillment type
+                if self.fulfillment_type.is_none() {
+                    return Err(new_application_error(
+                        "MISSING_FULFILLMENT_TYPE",
+                        "please select a fulfillment type to continue",
+                    ));
+                }
+
+                self.enter_completed_state().await
+            }
             State::PaymentInProgress(_) => self.enter_completed_state().await,
             _ => Err(new_invalid_state_error(
                 "must be in the PaymentInProgress or Completed state",
@@ -387,6 +402,24 @@ impl Checkout {
         ctx.update_checkout(self).await
     }
 
+    pub async fn initiate_order<C: Context + Send>(
+        &mut self,
+        ctx: &mut C,
+        payment_args: Option<<C as PaymentProcessor>::ProcessArgs>,
+    ) -> Result<(), Error> {
+        if !matches!(self.state, State::ItemsConfirmed(_)) {
+            return Err(new_invalid_state_error(
+                "must be in the ItemsConfirmed state",
+            ));
+        }
+
+        self.ensure_completed_state().await?;
+        let order = Order::new_from_checkout_missing_payment(ctx, self, payment_args).await?;
+        self.order_id = Some(order.id.clone());
+
+        ctx.update_checkout(self).await
+    }
+
     pub async fn revert<C: Context + Send>(&mut self, ctx: &mut C) -> Result<(), Error> {
         match self.state {
             State::Shopping => Ok(()),
@@ -408,7 +441,15 @@ impl Checkout {
     }
 
     pub async fn complete<C: Context + Send>(&mut self, ctx: &mut C) -> Result<(), Error> {
+        if !matches!(self.state, State::PaymentInProgress(_)) {
+            return Err(new_invalid_state_error(
+                "must be in the PaymentInProgress state",
+            ));
+        }
+
         self.ensure_completed_state().await?;
+        let order = Order::new_from_checkout(ctx, self).await?;
+        self.order_id = Some(order.id.clone());
 
         ctx.update_checkout(self).await
     }
