@@ -4,6 +4,8 @@ use crate::checkout::CheckoutStore;
 use crate::error::Error;
 use crate::inventory::InventoryController;
 use crate::invoice::InvoiceCalculator;
+use crate::order::OrderEvents;
+use crate::order::OrderStore;
 use crate::payment::PaymentProcessor;
 use crate::payment::PaymentStore;
 use crate::price::PriceCalculator;
@@ -22,10 +24,12 @@ pub trait Context:
     + CheckoutStore
     + PaymentProcessor
     + PaymentStore
+    + OrderStore
+    + OrderEvents
 {
 }
 
 #[async_trait]
-pub trait ContextProvider<'a, T: Context> {
-    async fn new_context(&'a self) -> Result<T, Error>;
+pub trait ContextProvider<T: Context> {
+    async fn new_context(&self) -> Result<T, Error>;
 }
