@@ -93,6 +93,16 @@ pub async fn process_handler<C: Context + Send>(ctx: &mut C, id: &str) -> Result
             let mut checkout = maybe_co.unwrap();
             process(ctx, &mut payment, Initiator::Checkout(&mut checkout)).await
         }
+        InitiatorId::Order(od_id) => {
+            let maybe_od = ctx.get_order_for_update(od_id).await?;
+            if maybe_od.is_none() {
+                // this isn't supposed to happen. maybe log the error and move on
+                return Ok(());
+            }
+
+            let mut order = maybe_od.unwrap();
+            process(ctx, &mut payment, Initiator::Order(&mut order)).await
+        }
     }
 }
 
@@ -123,6 +133,15 @@ pub async fn cancel_handler<C: Context + Send>(
             let mut checkout = maybe_co.unwrap();
             cancel(ctx, &mut payment, Initiator::Checkout(&mut checkout)).await?;
         }
+        InitiatorId::Order(od_id) => {
+            let maybe_od = ctx.get_order_for_update(od_id).await?;
+            if maybe_od.is_none() {
+                return Err(new_not_found_error("the order could not be found"));
+            }
+
+            let mut order = maybe_od.unwrap();
+            cancel(ctx, &mut payment, Initiator::Order(&mut order)).await?;
+        }
     }
 
     Ok(payment)
