@@ -195,15 +195,11 @@ async fn new<'a, C: Context + Send>(
         match payment.state {
             PaymentState::Succeeded => match initiator {
                 Initiator::Checkout(co) => co.complete(ctx).await?,
-                Initiator::Order(_) => {
-                    // TODO: transition order to PendingConfirmation or Confirmed
-                }
+                Initiator::Order(od) => od.prompt_confirmation(ctx).await?,
             },
             PaymentState::Cancelled => match initiator {
                 Initiator::Checkout(co) => co.revert(ctx).await?,
-                Initiator::Order(_) => {
-                    // TODO: transition order to Cancelled
-                }
+                Initiator::Order(od) => od.cancel(ctx).await?,
             },
             _ => {}
         }
@@ -276,17 +272,11 @@ pub async fn process<'a, C: Context + Send>(
             match payment.state {
                 PaymentState::Succeeded => match initiator {
                     Initiator::Checkout(co) => co.complete(ctx).await,
-                    Initiator::Order(_) => {
-                        // TODO: transition order to PendingConfirmation or Confirmed
-                        Ok(())
-                    }
+                    Initiator::Order(od) => od.prompt_confirmation(ctx).await,
                 },
                 PaymentState::Cancelled => match initiator {
                     Initiator::Checkout(co) => co.revert(ctx).await,
-                    Initiator::Order(_) => {
-                        // TODO: transition order to Cancelled
-                        Ok(())
-                    }
+                    Initiator::Order(od) => od.cancel(ctx).await,
                 },
                 _ => Ok(()),
             }
@@ -320,10 +310,7 @@ pub async fn cancel<'a, C: Context + Send>(
 
             match initiator {
                 Initiator::Checkout(co) => co.revert(ctx).await,
-                Initiator::Order(_) => {
-                    // TODO: transition order to Cancelled
-                    Ok(())
-                }
+                Initiator::Order(od) => od.cancel(ctx).await,
             }
         }
         PaymentState::Processing(_) => Err(new_application_error(
