@@ -1,5 +1,5 @@
 use async_trait::async_trait;
-use dropshot::{ApiDescription, HttpServerStarter, ServerContext};
+use dropshot::{ApiDescription, ServerContext};
 use schemars::JsonSchema;
 use serde::{Deserialize, Serialize};
 
@@ -29,8 +29,8 @@ pub struct AddOrRemoveItemParams {
 }
 
 #[async_trait]
-pub trait Server<'a, T: Context>: ContextProvider<'a, T> {
+pub trait Server<T: Context>: ContextProvider<T> {
     type Context: ServerContext;
     fn api_description() -> ApiDescription<Self::Context>;
-    async fn new_server(self, bind_address: &str) -> Result<HttpServerStarter<Self::Context>, Error>;
+    async fn start(self) -> Result<(), Error>;
 }
